'use strict';

describe('SCHEMAcat', function() {
	describe('Routing scenario', function() {

		afterEach(function() {
				//load the default initial page
				browser().navigateTo('/schemacat/site/index.html#/workspace/page/1/?schemaIndex=0');
		});

		it('should jump to "/workspace/page/1/?schemaIndex=0" when loading <serverURL>/schemacat/site/', function() {
			browser().navigateTo('/schemacat/site/');
			expect(browser().location().url()).toBe('/workspace/page/1/?schemaIndex=0');
		});

		it('should jump to "/workspace/page/1/?schemaIndex=0" when location path and hash are empty', function() {
			browser().navigateTo('/schemacat/site/index.html');
			expect(browser().location().url()).toBe('/workspace/page/1/?schemaIndex=0');
		});

		it('should have a working /workspace/<page> route', function() {
			browser().navigateTo('#/workspace/page/1/');
			expect(browser().location().path()).toBe('/workspace/page/1/');
		});

		it('should have a working /about route', function() {
			browser().navigateTo('#/about');
			expect(browser().location().url()).toBe('/about');
		});

		it('should have a working /contact route', function() {
			browser().navigateTo('#/contact');
			expect(browser().location().url()).toBe('/contact');
		});

		it('should have a working /accountDetails route', function() {
			browser().navigateTo('#/accountDetails');
			expect(browser().location().url()).toBe('/accountDetails');
		});

		it('should jump to a different schema when schemaIndex changes', function() {
			browser().navigateTo('#/workspace/page/1/?schemaIndex=1');
			expect(browser().location().search()).toEqual({schemaIndex: '1'});
			browser().navigateTo('#/workspace/page/1/?schemaIndex=2');
			expect(browser().location().search()).toEqual({schemaIndex: '2'});
		});

		it('Should login as administrator', function() {
			element('li[title="userMenu"] a:has(span[data-ng-bind="displayName"])').click();
			element('li[title="userMenu"] span:contains("Login as a different user...")').click();
			input('loginNameInput.text').enter('Menzo.Windhouwer_mpi.nl@clarin.eu');
			alertOK();
			element('#login').click();
			sleep(0.25);
			expect(browser().location().url()).toBe('/workspace/page/1/?schemaIndex=0');
			expect(element('li[title="userMenu"] span[data-ng-bind="displayName"]').text()).toBe('Menzo Windhouwer');
		});

		it('should jump to a different page when page path changes', function() {
			browser().navigateTo('#/workspace/page/2/');
			expect(browser().location().path()).toBe('/workspace/page/2/');
		});

		it('Should login as guest', function() {
			element('li[title="userMenu"] a:has(span[data-ng-bind="displayName"])').click();
			element('li[title="userMenu"] span:contains("Login as a different user...")').click();
			input('loginNameInput.text').enter('guest');
			alertOK();
			element('#login').click();
			sleep(0.25);
			expect(browser().location().url()).toBe('/workspace/page/1/?schemaIndex=0');
			expect(element('li[title="userMenu"] span[data-ng-bind="displayName"]').text()).toBe('guest');
		});
	});
	describe('Worspace scenarios', function() {
		describe('Routes scenario', function() {

			beforeEach(function() {
				//load the default initial page
				browser().navigateTo('/schemacat/site/index.html#/workspace/page/1/?schemaIndex=0');
			});
			it('should redirect to "/workspace/page/1/?schemaIndex=0" when page is invalid (page="a")', function() {
				browser().navigateTo('#/workspace/page/a/?schemaIndex=0');
				expect(browser().location().url()).toBe('/workspace/page/1/?schemaIndex=0');
			});

			it('should redirect to "/workspace/page/1/?schemaIndex=0" when page is out of range (page=3)', function() {
				browser().navigateTo('#/workspace/page/3/?schemaIndex=0');
				expect(browser().location().url()).toBe('/workspace/page/1/?schemaIndex=0');
			});

			it('should redirect to "/workspace/page/1/?schemaIndex=0" when page is out of range (page<1)', function() {
				browser().navigateTo('#/workspace/page/0/?schemaIndex=0');
				expect(browser().location().url()).toBe('/workspace/page/1/?schemaIndex=0');
			});

			it('should redirect to "/workspace/page/1/?schemaIndex=0" when schemaIndex is invalid (schemaIndex="a")', function() {
				browser().navigateTo('#/workspace/page/1/?schemaIndex=a');
				expect(browser().location().url()).toBe('/workspace/page/1/?schemaIndex=0');
			});

			it('should redirect to "/workspace/page/1/?schemaIndex=0" when schemaIndex is out of range (schemaIndex>9)', function() {
				browser().navigateTo('#/workspace/page/1/?schemaIndex=10');
				expect(browser().location().url()).toBe('/workspace/page/1/?schemaIndex=0');
			});

			it('should redirect to "/workspace/page/1/?schemaIndex=0" when schemaIndex is out of range (schemaIndex=-1)', function() {
				browser().navigateTo('#/workspace/page/1/?schemaIndex=-1');
				expect(browser().location().url()).toBe('/workspace/page/1/?schemaIndex=0');
			});

			it('should show the schema with index=0 if schemaIndex is not supplied on the route', function() {
				browser().navigateTo('#/workspace/page/1/');
				expect(element('#schema-name').html()).not().toEqual('');
				expect(element('#schema-name').css('visibility')).toBe('visible');
			});

		});
		describe('Change schema scenario', function() {

			beforeEach(function() {
				//load the default initial page
				browser().navigateTo('/schemacat/site/index.html#/workspace/page/1/?schemaIndex=0');
			});

			it('should show the clicked schema and update route (animated)', function() {
				element('#workspace-main-row table tbody tr:eq(1)').click();

				expect(browser().location().search()).toEqual({schemaIndex: '1'});
				expect(element('#schema-name').html()).not().toEqual('');
				expect(element('#schema-name').css('visibility')).toBe('visible');
				expect(element('#schema-div').attr('class')).toContain('animated fadeInUp');

				element('#workspace-main-row table tbody tr:eq(0)').click();

				expect(browser().location().search()).toEqual({schemaIndex: '0'});
				expect(element('#schema-name').html()).not().toEqual('');
				expect(element('#schema-name').css('visibility')).toBe('visible');
				expect(element('#schema-div').attr('class')).toContain('animated fadeInDown');
			});

			it('should show the clicked schema and update route (not animated)', function() {
				browser().navigateTo('/schemacat/site/index.html#/workspace/page/1/?schemaIndex=0&animationsDisabled');
				element('#workspace-main-row table tbody tr:eq(1)').click();

				expect(browser().location().search()).toEqual({schemaIndex: '1', animationsDisabled: true});
				expect(element('#schema-name').html()).not().toEqual('');
				expect(element('#schema-name').css('visibility')).toBe('visible');
				expect(element('#schema-div').attr('class')).not().toContain('animated');

				element('#workspace-main-row table tbody tr:eq(0)').click();

				expect(browser().location().search()).toEqual({schemaIndex: '0', animationsDisabled: true});
				expect(element('#schema-name').html()).not().toEqual('');
				expect(element('#schema-name').css('visibility')).toBe('visible');
				expect(element('#schema-div').attr('class')).not().toContain('animated');
			});

		});
		describe('Open schema file scenario', function() {
			beforeEach(function() {
				//load the default initial page
				browser().navigateTo('/schemacat/site/index.html#/workspace/page/1/?schemaIndex=0');
			});
			it('should navigate to the schema file url', function() {
				element('#files-list a:contains("Open"):eq(0)').click();
				expect(browser().window().href()).toContain('/schemacat/schemas/');
				expect(browser().window().href()).toContain('/files/');
			});
		});
	});
	describe('Contact page scenarios', function() {
		describe('Load page scenario (animated)', function() {
			beforeEach(function() {
				browser().navigateTo('/schemacat/site/index.html#/workspace/page/1/?schemaIndex=0');
			});

			it('should load the contact page and slide it from the right when url changes from the workspace page', function() {
				browser().navigateTo('#/contact');
				expect(browser().location().url()).toBe('/contact');
				expect(element('#contact').html()).not().toEqual('');
				expect(element('#contact').attr('class')).toContain('animated slideInRight');
			});

			it('should load the contact page and slide it from the right when url changes from the about page', function() {
				browser().navigateTo('#/about');
				browser().navigateTo('#/contact');
				expect(browser().location().url()).toBe('/contact');
				expect(element('#contact').html()).not().toEqual('');
				expect(element('#contact').attr('class')).toContain('animated slideInRight');
			});

			it('should load the contact page and fade it down when url changes from the accountDetails page', function() {
				browser().navigateTo('#/accountDetails');
				browser().navigateTo('#/contact');
				expect(browser().location().url()).toBe('/contact');
				expect(element('#contact').html()).not().toEqual('');
				expect(element('#contact').attr('class')).toContain('animated fadeInDown');
			});

			it('should load the contact page and fade it down when reload is clicked', function() {
				browser().navigateTo('#/contact');
				browser().reload();
				expect(browser().location().url()).toBe('/contact');
				expect(element('#contact').html()).not().toEqual('');
				expect(element('#contact').attr('class')).toContain('animated fadeInDown');
			});

			it('should load the contact page and fade it down when landing on this page from an external location', function() {
				browser().navigateTo('/');
				browser().navigateTo('/schemacat/site/index.html#/contact');
				expect(browser().location().url()).toBe('/contact');
				expect(element('#contact').html()).not().toEqual('');
				expect(element('#contact').attr('class')).toContain('animated fadeInDown');
			});
		});
		describe('Load page scenario (not animated)', function() {
			beforeEach(function() {
				browser().navigateTo('/schemacat/site/index.html#/workspace/page/1/?schemaIndex=0&animationsDisabled');
			});

			it('should load the contact page without animations when url changes from the workspace page', function() {
				browser().navigateTo('#/contact?animationsDisabled');
				expect(browser().location().url()).toBe('/contact?animationsDisabled');
				expect(element('#contact').html()).not().toEqual('');
				expect(element('#contact').attr('class')).not().toContain('animated');
			});

			it('should load the contact page without animations when url changes from the about page', function() {
				browser().navigateTo('#/about?animationsDisabled');
				browser().navigateTo('#/contact?animationsDisabled');
				expect(browser().location().url()).toBe('/contact?animationsDisabled');
				expect(element('#contact').html()).not().toEqual('');
				expect(element('#contact').attr('class')).not().toContain('animated');
			});

			it('should load the contact page without animations when url changes from the accountDetails page', function() {
				browser().navigateTo('#/accountDetails?animationsDisabled');
				browser().navigateTo('#/contact?animationsDisabled');
				expect(browser().location().url()).toBe('/contact?animationsDisabled');
				expect(element('#contact').html()).not().toEqual('');
				expect(element('#contact').attr('class')).not().toContain('animated');
			});

			it('should load the contact page without animations when reload is clicked', function() {
				browser().navigateTo('#/contact?animationsDisabled');
				browser().reload();
				expect(browser().location().url()).toBe('/contact?animationsDisabled');
				expect(element('#contact').html()).not().toEqual('');
				expect(element('#contact').attr('class')).not().toContain('animated');
			});

			it('should load the contact page without animations when landing on this page from an external location', function() {
				browser().navigateTo('/');
				browser().navigateTo('/schemacat/site/index.html#/contact?animationsDisabled');
				expect(browser().location().url()).toBe('/contact?animationsDisabled');
				expect(element('#contact').html()).not().toEqual('');
				expect(element('#contact').attr('class')).not().toContain('animated');
			});
		});
	});
});